@@ -19,7 +19,7 @@ use std::marker::PhantomData;
 use std::pin::Pin;
 use std::sync::{Arc, Mutex};
 
-use super::async_frame::AsyncGcFrame;
+use super::{async_frame::AsyncGcFrame, yield_task};
 
 pub(crate) struct TaskState<'frame, 'data> {
     completed: bool,
@@ -61,11 +61,11 @@ impl<'frame, 'data> JuliaFuture<'frame, 'data> {
 
             let global = frame.global();
             let task = Module::main(global)
-                .submodule_ref("Jlrs")?
+                .submodule_ref("JlrsMultitask")?
                 .wrapper_unchecked()
                 .function_ref("asynccall")?
                 .wrapper_unchecked()
-                .call(frame, &mut vals)?
+                .call(&mut *frame, &mut vals)?
                 .map_err(|e| {
                     let msg = e.display_string_or(CANNOT_DISPLAY_VALUE);
                     JlrsError::Exception {
@@ -82,6 +82,7 @@ impl<'frame, 'data> JuliaFuture<'frame, 'data> {
                 }
             }
 
+            yield_task(frame);
             Ok(JuliaFuture { shared_state })
         }
     }
@@ -114,11 +115,65 @@ impl<'frame, 'data> JuliaFuture<'frame, 'data> {
 
             let global = frame.global();
             let task = Module::main(global)
-                .submodule_ref("Jlrs")?
+                .submodule_ref("JlrsMultitask")?
+                .wrapper_unchecked()
+                .function_ref("scheduleasynclocal")?
+                .wrapper_unchecked()
+                .call(&mut *frame, &mut vals)?
+                .map_err(|e| {
+                    let msg = e.display_string_or(CANNOT_DISPLAY_VALUE);
+                    JlrsError::Exception {
+                        msg: format!("scheduleasynclocal threw an exception: {}", msg),
+                    }
+                })?
+                .cast_unchecked::<Task>();
+
+            {
+                let locked = shared_state.lock();
+                match locked {
+                    Ok(mut data) => data.task = Some(task),
+                    _ => exception("Cannot set task".into())?,
+                }
+            }
+
+            yield_task(frame);
+            Ok(JuliaFuture { shared_state })
+        }
+    }
+
+    pub(crate) fn new_main<'value, V>(
+        frame: &mut AsyncGcFrame<'frame>,
+        func: Value,
+        mut values: V,
+    ) -> JlrsResult<Self>
+    where
+        V: AsMut<[Value<'value, 'data>]>,
+    {
+        let shared_state = Arc::new(Mutex::new(TaskState {
+            completed: false,
+            waker: None,
+            task: None,
+            _marker: PhantomData,
+        }));
+
+        unsafe {
+            let values = values.as_mut();
+            let state_ptr = Arc::into_raw(shared_state.clone()) as *mut c_void;
+            let state_ptr_boxed = Value::new(&mut *frame, state_ptr)?;
+
+            let mut vals: SmallVec<[Value; MAX_SIZE]> = SmallVec::with_capacity(2 + values.len());
+
+            vals.push(func);
+            vals.push(state_ptr_boxed);
+            vals.extend_from_slice(values);
+
+            let global = frame.global();
+            let task = Module::main(global)
+                .submodule_ref("JlrsMultitask")?
                 .wrapper_unchecked()
                 .function_ref("scheduleasync")?
                 .wrapper_unchecked()
-                .call(frame, &mut vals)?
+                .call(&mut *frame, &mut vals)?
                 .map_err(|e| {
                     let msg = e.display_string_or(CANNOT_DISPLAY_VALUE);
                     JlrsError::Exception {
@@ -135,6 +190,7 @@ impl<'frame, 'data> JuliaFuture<'frame, 'data> {
                 }
             }
 
+            yield_task(frame);
             Ok(JuliaFuture { shared_state })
         }
     }
@@ -167,12 +223,12 @@ impl<'frame, 'data> JuliaFuture<'frame, 'data> {
 
             let global = frame.global();
             let task = Module::main(global)
-                .submodule_ref("Jlrs")?
+                .submodule_ref("JlrsMultitask")?
                 .wrapper_unchecked()
                 .function_ref("asynccall")?
                 .wrapper_unchecked()
                 .with_keywords(func.keywords())?
-                .call(frame, &mut vals)?
+                .call(&mut *frame, &mut vals)?
                 .map_err(|e| {
                     let msg = e.display_string_or(CANNOT_DISPLAY_VALUE);
                     JlrsError::Exception {
@@ -189,6 +245,8 @@ impl<'frame, 'data> JuliaFuture<'frame, 'data> {
                 }
             }
 
+            yield_task(frame);
+
             Ok(JuliaFuture { shared_state })
         }
     }
@@ -221,12 +279,68 @@ impl<'frame, 'data> JuliaFuture<'frame, 'data> {
 
             let global = frame.global();
             let task = Module::main(global)
-                .submodule_ref("Jlrs")?
+                .submodule_ref("JlrsMultitask")?
+                .wrapper_unchecked()
+                .function_ref("scheduleasynclocal")?
+                .wrapper_unchecked()
+                .with_keywords(func.keywords())?
+                .call(&mut *frame, &mut vals)?
+                .map_err(|e| {
+                    let msg = e.display_string_or(CANNOT_DISPLAY_VALUE);
+                    JlrsError::Exception {
+                        msg: format!("scheduleasynclocal threw an exception: {}", msg),
+                    }
+                })?
+                .cast_unchecked::<Task>();
+
+            {
+                let locked = shared_state.lock();
+                match locked {
+                    Ok(mut data) => data.task = Some(task),
+                    _ => exception("Cannot set task".into())?,
+                }
+            }
+
+            yield_task(frame);
+
+            Ok(JuliaFuture { shared_state })
+        }
+    }
+
+    pub(crate) fn new_main_with_keywords<'value, V>(
+        frame: &mut AsyncGcFrame<'frame>,
+        func: WithKeywords,
+        mut values: V,
+    ) -> JlrsResult<Self>
+    where
+        V: AsMut<[Value<'value, 'data>]>,
+    {
+        let shared_state = Arc::new(Mutex::new(TaskState {
+            completed: false,
+            waker: None,
+            task: None,
+            _marker: PhantomData,
+        }));
+
+        unsafe {
+            let values = values.as_mut();
+            let state_ptr = Arc::into_raw(shared_state.clone()) as *mut c_void;
+            let state_ptr_boxed = Value::new(&mut *frame, state_ptr)?;
+
+            let mut vals: SmallVec<[Value; MAX_SIZE]> = SmallVec::with_capacity(2 + values.len());
+
+            vals.push(func.function());
+            vals.push(state_ptr_boxed);
+            vals.extend_from_slice(values);
+
+            let global = frame.global();
+            let task = Module::main(global)
+                .submodule_ref("JlrsMultitask")?
                 .wrapper_unchecked()
                 .function_ref("scheduleasync")?
                 .wrapper_unchecked()
                 .with_keywords(func.keywords())?
-                .call(frame, &mut vals)?
+                .call(&mut *frame, &mut vals)?
                 .map_err(|e| {
                     let msg = e.display_string_or(CANNOT_DISPLAY_VALUE);
                     JlrsError::Exception {
@@ -243,6 +357,8 @@ impl<'frame, 'data> JuliaFuture<'frame, 'data> {
                 }
             }
 
+            yield_task(frame);
+
             Ok(JuliaFuture { shared_state })
         }
     }
@@ -286,6 +402,7 @@ impl<'frame, 'data> Future for JuliaFuture<'frame, 'data> {
 
 // This function is set as a constant in `Main.Jlrs` and called using `ccall` to indicate a task has
 // completed.
+#[cfg(any(feature = "async-std-rt", feature = "tokio-rt"))]
 pub(crate) unsafe extern "C" fn wake_task(state: *const Mutex<TaskState>) {
     let state = Arc::from_raw(state);
     let shared_state = state.lock();
