@@ -1,6 +1,5 @@
 //! Traits for checking layout compatibility and enforcing layout requirements.
 
 pub mod bits_union;
-pub mod julia_type;
-pub mod julia_typecheck;
+pub mod typecheck;
 pub mod valid_layout;
