@@ -31,14 +31,18 @@
 //! [`Scope::result_scope`]: crate::memory::scope::Scope::result_scope
 //! [`ScopeExt::scope`]: crate::memory::scope::ScopeExt::scope
 
-use super::{mode::Mode, stack_page::StackPage};
-use crate::{private::Private, CCall};
-use jl_sys::jl_value_t;
-use std::{
-    ffi::c_void,
-    marker::PhantomData,
-    ptr::{null_mut, NonNull},
-};
+use super::{mode::Mode, reusable_slot::ReusableSlot, stack_page::StackPage};
+#[cfg(feature = "ccall")]
+use crate::error::JlrsError;
+use crate::{error::JlrsResult, private::Private};
+
+#[cfg(feature = "ccall")]
+use crate::ccall::CCall;
+
+use jl_sys::{jl_get_current_task, jl_task_t, jl_value_t};
+#[cfg(feature = "ccall")]
+use std::marker::PhantomData;
+use std::{cell::Cell, ffi::c_void, ptr::NonNull};
 
 pub(crate) const MIN_FRAME_CAPACITY: usize = 16;
 
@@ -48,21 +52,21 @@ pub(crate) const MIN_FRAME_CAPACITY: usize = 16;
 /// preallocate that number of slots. Frames created without slots will dynamically create new
 /// slots as needed. A frame's capacity is at least 16.
 pub struct GcFrame<'frame, M: Mode> {
-    pub(crate) raw_frame: &'frame mut [*mut c_void],
+    pub(crate) raw_frame: &'frame mut [Cell<*mut c_void>],
     pub(crate) page: Option<StackPage>,
-    pub(crate) n_roots: usize,
     pub(crate) mode: M,
 }
 
 impl<'frame, M: Mode> GcFrame<'frame, M> {
     /// Returns the number of values currently rooted in this frame.
     pub fn n_roots(&self) -> usize {
-        self.n_roots
+        self.raw_frame[0].get() as usize >> 2
     }
 
-    /// Returns the number of slots that are currently allocated to this frame.
-    pub fn n_slots(&self) -> usize {
-        self.raw_frame[0] as usize >> 1
+    #[doc(hidden)]
+    pub unsafe fn print_stack(&self) {
+        let last = jl_get_current_task().cast::<jl_task_t>();
+        jl_sys::jlrs_print_stack(NonNull::new_unchecked(last).as_ref().gcstack);
     }
 
     /// Returns the maximum number of slots this frame can use.
@@ -70,27 +74,9 @@ impl<'frame, M: Mode> GcFrame<'frame, M> {
         self.raw_frame.len() - 2
     }
 
-    /// Try to allocate `additional` slots in the current frame. Returns `true` on success, or
-    /// `false` if `self.n_slots() + additional > self.capacity()`.
-    #[must_use]
-    pub fn alloc_slots(&mut self, additional: usize) -> bool {
-        let slots = self.n_slots();
-        if additional + slots > self.capacity() {
-            return false;
-        }
-
-        for idx in slots + 2..slots + additional + 2 {
-            self.raw_frame[idx] = null_mut();
-        }
-
-        // The new number of slots doesn't  exceed the capacity, and the new slots have been cleared
-        unsafe { self.set_n_slots(slots + additional) }
-        true
-    }
-
     // Safety: this frame must be dropped in the same scope it has been created.
     pub(crate) unsafe fn nest<'nested>(&'nested mut self, capacity: usize) -> GcFrame<'nested, M> {
-        let used = self.n_slots() + 2;
+        let used = self.n_roots() + 2;
         let new_frame_size = MIN_FRAME_CAPACITY.max(capacity) + 2;
         let raw_frame = if self.page.is_some() {
             if new_frame_size <= self.page.as_ref().unwrap().size() {
@@ -106,26 +92,25 @@ impl<'frame, M: Mode> GcFrame<'frame, M> {
             self.page.as_mut().unwrap().as_mut()
         };
 
-        GcFrame::new(raw_frame, capacity, self.mode)
+        GcFrame::new(raw_frame, self.mode)
     }
 
     // Safety: this frame must be dropped in the same scope it has been created and raw_frame must
     // have 2 + slots capacity available.
-    pub(crate) unsafe fn new(raw_frame: &'frame mut [*mut c_void], slots: usize, mode: M) -> Self {
-        mode.push_frame(raw_frame, slots, Private);
+    pub(crate) unsafe fn new(raw_frame: &'frame mut [Cell<*mut c_void>], mode: M) -> Self {
+        mode.push_frame(raw_frame, Private);
 
         GcFrame {
             raw_frame,
             page: None,
-            n_roots: 0,
             mode,
         }
     }
 
     // Safety: capacity >= n_slots
-    pub(crate) unsafe fn set_n_slots(&mut self, n_slots: usize) {
-        debug_assert!(self.capacity() >= n_slots);
-        *self.raw_frame.get_unchecked_mut(0) = (n_slots << 1) as _;
+    pub(crate) unsafe fn set_n_roots(&mut self, n_roots: usize) {
+        debug_assert!(self.capacity() >= n_roots);
+        self.raw_frame.get_unchecked_mut(0).set((n_roots << 2) as _);
     }
 
     // Safety: capacity > n_roots
@@ -133,10 +118,10 @@ impl<'frame, M: Mode> GcFrame<'frame, M> {
         debug_assert!(self.n_roots() < self.capacity());
 
         let n_roots = self.n_roots();
-        *self.raw_frame.get_unchecked_mut(n_roots + 2) = value.cast().as_ptr();
-        if n_roots == self.n_slots() {
-            self.set_n_slots(n_roots + 1);
-        }
+        self.raw_frame
+            .get_unchecked_mut(n_roots + 2)
+            .set(value.cast().as_ptr());
+        self.set_n_roots(n_roots + 1);
     }
 }
 
@@ -150,8 +135,10 @@ impl<'frame, M: Mode> Drop for GcFrame<'frame, M> {
 /// A `NullFrame` can be used if you call Rust from Julia through `ccall` and want to borrow array
 /// data but not perform any allocations. It can't be used to created a nested scope or root a
 /// newly allocated value. If you try to do so a `JlrsError::NullFrame` is returned.
+#[cfg(feature = "ccall")]
 pub struct NullFrame<'frame>(PhantomData<&'frame ()>);
 
+#[cfg(feature = "ccall")]
 impl<'frame> NullFrame<'frame> {
     pub(crate) unsafe fn new(_: &'frame mut CCall) -> Self {
         NullFrame(PhantomData)
@@ -168,65 +155,56 @@ pub trait Frame<'frame>: private::Frame<'frame> {
         self
     }
 
-    /// Reserve `additional` slots in the current frame. Returns `true` on success, or `false` if
-    /// `self.n_slots() + additional > self.capacity()`.
-    #[must_use]
-    fn alloc_slots(&mut self, additional: usize) -> bool;
+    fn reusable_slot(&mut self) -> JlrsResult<ReusableSlot<'frame>>;
 
     /// Returns the number of values currently rooted in this frame.
     fn n_roots(&self) -> usize;
 
-    /// Returns the number of slots that are currently allocated to this frame.
-    fn n_slots(&self) -> usize;
-
     /// Returns the maximum number of slots this frame can use.
     fn capacity(&self) -> usize;
 }
 
 impl<'frame, M: Mode> Frame<'frame> for GcFrame<'frame, M> {
-    #[must_use]
-    fn alloc_slots(&mut self, additional: usize) -> bool {
-        self.alloc_slots(additional)
+    fn reusable_slot(&mut self) -> JlrsResult<ReusableSlot<'frame>> {
+        ReusableSlot::new(self)
     }
 
     fn n_roots(&self) -> usize {
         self.n_roots()
     }
 
-    fn n_slots(&self) -> usize {
-        self.n_slots()
-    }
-
     fn capacity(&self) -> usize {
         self.capacity()
     }
 }
 
+#[cfg(feature = "ccall")]
 impl<'frame> Frame<'frame> for NullFrame<'frame> {
-    #[must_use]
-    fn alloc_slots(&mut self, _additional: usize) -> bool {
-        false
+    fn reusable_slot(&mut self) -> JlrsResult<ReusableSlot<'frame>> {
+        Err(JlrsError::NullFrame)?
     }
 
     fn n_roots(&self) -> usize {
         0
     }
 
-    fn n_slots(&self) -> usize {
-        0
-    }
-
     fn capacity(&self) -> usize {
         0
     }
 }
 
 pub(crate) mod private {
-    use std::ptr::NonNull;
+    use std::{
+        ffi::c_void,
+        ptr::{null_mut, NonNull},
+    };
 
     use crate::memory::frame::GcFrame;
+    #[cfg(feature = "ccall")]
     use crate::memory::frame::NullFrame;
-    use crate::memory::mode::{Mode, Sync};
+    use crate::memory::mode::Mode;
+    #[cfg(feature = "ccall")]
+    use crate::memory::mode::Sync;
     use crate::memory::root_pending::RootPending;
     use crate::wrappers::ptr::private::Wrapper;
     use crate::wrappers::ptr::value::Value;
@@ -247,6 +225,8 @@ pub(crate) mod private {
             _: Private,
         ) -> Result<Value<'frame, 'data>, AllocError>;
 
+        unsafe fn reserve_slot(&mut self, _: Private) -> JlrsResult<*mut *mut c_void>;
+
         unsafe fn nest<'nested>(
             &'nested mut self,
             capacity: usize,
@@ -329,6 +309,22 @@ pub(crate) mod private {
             Ok(Value::wrap_non_null(value, Private))
         }
 
+        unsafe fn reserve_slot(&mut self, _: Private) -> JlrsResult<*mut *mut c_void> {
+            let n_roots = self.n_roots();
+            if n_roots == self.capacity() {
+                Err(JlrsError::alloc_error(AllocError::FrameOverflow(
+                    1, n_roots,
+                )))?;
+            }
+
+            self.raw_frame
+                .get_unchecked_mut(n_roots + 2)
+                .set(null_mut());
+            self.set_n_roots(n_roots + 1);
+
+            Ok(self.raw_frame.get_unchecked_mut(n_roots + 2).as_ptr())
+        }
+
         unsafe fn nest<'nested>(
             &'nested mut self,
             capacity: usize,
@@ -436,6 +432,7 @@ pub(crate) mod private {
         }
     }
 
+    #[cfg(feature = "ccall")]
     impl<'frame> Frame<'frame> for NullFrame<'frame> {
         type Mode = Sync;
 
@@ -447,6 +444,10 @@ pub(crate) mod private {
             Err(AllocError::FrameOverflow(1, 0))
         }
 
+        unsafe fn reserve_slot(&mut self, _: Private) -> JlrsResult<*mut *mut c_void> {
+            Err(JlrsError::NullFrame)?
+        }
+
         unsafe fn nest<'nested>(
             &'nested mut self,
             _capacity: usize,
@@ -537,3 +538,121 @@ pub(crate) mod private {
         }
     }
 }
+
+#[cfg(test)]
+#[cfg(feature = "sync-rt")]
+mod tests {
+    use crate::{
+        memory::{frame::GcFrame, mode, stack_page::StackPage},
+        util,
+        wrappers::ptr::value::Value,
+    };
+
+    #[test]
+    fn min_stack_pack_size() {
+        let mut page = StackPage::new(0);
+        assert_eq!(page.as_mut().len(), 64);
+    }
+
+    #[test]
+    fn create_base_frame() {
+        util::JULIA.with(|julia| unsafe {
+            let mut julia = julia.borrow_mut();
+            let page = julia.get_page();
+            let page_size = page.size();
+
+            let frame = GcFrame::new(page.as_mut(), mode::Sync);
+            assert_eq!(frame.capacity(), page_size - 2);
+            assert_eq!(frame.n_roots(), 0);
+        })
+    }
+
+    #[test]
+    fn push_root() {
+        util::JULIA.with(|julia| unsafe {
+            let mut julia = julia.borrow_mut();
+            let page = julia.get_page();
+            let page_size = page.size();
+            let mut frame = GcFrame::new(page.as_mut(), mode::Sync);
+            let _value = Value::new(&mut frame, 1usize).unwrap();
+
+            assert_eq!(frame.capacity(), page_size - 2);
+            assert_eq!(frame.n_roots(), 1);
+        })
+    }
+
+    #[test]
+    fn push_too_many_roots() {
+        util::JULIA.with(|julia| unsafe {
+            let mut julia = julia.borrow_mut();
+            let page = julia.get_page();
+            let page_size = page.size();
+            let mut frame = GcFrame::new(page.as_mut(), mode::Sync);
+
+            for _ in 0..page_size - 2 {
+                let _value = Value::new(&mut frame, 1usize).unwrap();
+            }
+
+            assert_eq!(frame.capacity(), page_size - 2);
+            assert_eq!(frame.n_roots(), page_size - 2);
+
+            assert!(Value::new(&mut frame, 1usize).is_err());
+        })
+    }
+
+    #[test]
+    fn push_new_frame() {
+        util::JULIA.with(|julia| unsafe {
+            let mut julia = julia.borrow_mut();
+            let page = julia.get_page();
+            let page_size = page.size();
+            let mut frame = GcFrame::new(page.as_mut(), mode::Sync);
+
+            {
+                let nested = frame.nest(0);
+                let capacity = nested.capacity();
+                assert_eq!(capacity, page_size - 4);
+            }
+        })
+    }
+
+    #[test]
+    fn push_large_new_frame() {
+        util::JULIA.with(|julia| unsafe {
+            let mut julia = julia.borrow_mut();
+            let page = julia.get_page();
+            let page_size = page.size();
+            let mut frame = GcFrame::new(page.as_mut(), mode::Sync);
+
+            {
+                let nested = frame.nest(2 * page_size);
+                let capacity = nested.capacity();
+                let n_roots = nested.n_roots();
+                assert_eq!(capacity, 2 * page_size);
+                assert_eq!(n_roots, 0);
+            }
+        })
+    }
+
+    #[test]
+    fn reuse_large_page() {
+        util::JULIA.with(|julia| unsafe {
+            let mut julia = julia.borrow_mut();
+            let page = julia.get_page();
+            let page_size = page.size();
+            let mut frame = GcFrame::new(page.as_mut(), mode::Sync);
+
+            {
+                frame.nest(2 * page_size);
+            }
+
+            {
+                let nested = frame.nest(0);
+                let capacity = nested.capacity();
+                let n_roots = nested.n_roots();
+                assert_eq!(capacity, 2 * page_size);
+                assert_eq!(n_roots, 0);
+            }
+        })
+    }
+}
